@@ -1,10 +1,16 @@
 use std::fmt::Debug;
+use std::marker::PhantomData;
 
 use super::OptValue;
 use crate::err::Result;
 use crate::set::Set;
 use crate::uid::Uid;
 
+/// The callback type of option.
+///
+/// Since rust has a lot of restrict on reference.
+/// So we can't store block code into option itself of [`Set`](crate::set::Set).
+/// Instead we put the callback code into [`Parser`](crate::parser::Parser).
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub enum CallbackType {
     Opt,
@@ -234,28 +240,29 @@ impl From<Box<dyn MainMutCallback>> for Callback {
     }
 }
 
-pub struct SimpleOptCallback<
-    T: 'static + FnMut(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>,
->(T);
+pub struct SimpleOptCallback<'a, T: 'a + Fn(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>>(
+    T,
+    PhantomData<&'a T>,
+);
 
-impl<T: 'static + FnMut(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>> SimpleOptCallback<T> {
+impl<'a, T: 'a + Fn(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>> SimpleOptCallback<'a, T> {
     pub fn new(cb: T) -> Self {
-        Self(cb)
+        Self(cb, PhantomData::default())
     }
 }
 
-impl<T: 'static + FnMut(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>> Debug
-    for SimpleOptCallback<T>
+impl<'a, T: 'a + Fn(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>> Debug
+    for SimpleOptCallback<'a, T>
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("SimpleOptCallback")
-            .field("FnMut", &String::from("..."))
+            .field("Fn", &String::from("..."))
             .finish()
     }
 }
 
-impl<T: 'static + FnMut(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>> OptCallback
-    for SimpleOptCallback<T>
+impl<'a, T: 'a + Fn(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>> OptCallback
+    for SimpleOptCallback<'a, T>
 {
     fn call(&mut self, uid: Uid, set: &dyn Set, value: OptValue) -> Result<Option<OptValue>> {
         self.0(uid, set, value)
@@ -263,19 +270,20 @@ impl<T: 'static + FnMut(Uid, &dyn Set, OptValue) -> Result<Option<OptValue>>> Op
 }
 
 pub struct SimpleOptMutCallback<
-    T: 'static + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>,
->(T);
+    'a,
+    T: 'a + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>,
+>(T, PhantomData<&'a T>);
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>>
-    SimpleOptMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>>
+    SimpleOptMutCallback<'a, T>
 {
     pub fn new(cb: T) -> Self {
-        Self(cb)
+        Self(cb, PhantomData::default())
     }
 }
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>> Debug
-    for SimpleOptMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>> Debug
+    for SimpleOptMutCallback<'a, T>
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("SimpleOptMutCallback")
@@ -284,8 +292,8 @@ impl<T: 'static + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>
     }
 }
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>> OptMutCallback
-    for SimpleOptMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>> OptMutCallback
+    for SimpleOptMutCallback<'a, T>
 {
     fn call(&mut self, uid: Uid, set: &mut dyn Set, value: OptValue) -> Result<Option<OptValue>> {
         self.0(uid, set, value)
@@ -293,29 +301,30 @@ impl<T: 'static + FnMut(Uid, &mut dyn Set, OptValue) -> Result<Option<OptValue>>
 }
 
 pub struct SimplePosCallback<
-    T: 'static + FnMut(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>,
->(T);
+    'a,
+    T: 'a + Fn(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>,
+>(T, PhantomData<&'a T>);
 
-impl<T: 'static + FnMut(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>>
-    SimplePosCallback<T>
+impl<'a, T: 'a + Fn(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>>
+    SimplePosCallback<'a, T>
 {
     pub fn new(cb: T) -> Self {
-        Self(cb)
+        Self(cb, PhantomData::default())
     }
 }
 
-impl<T: 'static + FnMut(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>> Debug
-    for SimplePosCallback<T>
+impl<'a, T: 'a + Fn(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>> Debug
+    for SimplePosCallback<'a, T>
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("SimplePosCallback")
-            .field("FnMut", &String::from("..."))
+            .field("Fn", &String::from("..."))
             .finish()
     }
 }
 
-impl<T: 'static + FnMut(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>> PosCallback
-    for SimplePosCallback<T>
+impl<'a, T: 'a + Fn(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>> PosCallback
+    for SimplePosCallback<'a, T>
 {
     fn call(
         &mut self,
@@ -330,19 +339,20 @@ impl<T: 'static + FnMut(Uid, &dyn Set, &str, u64, OptValue) -> Result<Option<Opt
 }
 
 pub struct SimplePosMutCallback<
-    T: 'static + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>,
->(T);
+    'a,
+    T: 'a + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>,
+>(T, PhantomData<&'a T>);
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>>
-    SimplePosMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>>
+    SimplePosMutCallback<'a, T>
 {
     pub fn new(cb: T) -> Self {
-        Self(cb)
+        Self(cb, PhantomData::default())
     }
 }
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>> Debug
-    for SimplePosMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>> Debug
+    for SimplePosMutCallback<'a, T>
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("SimplePosMutCallback")
@@ -351,8 +361,8 @@ impl<T: 'static + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option
     }
 }
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>>
-    PosMutCallback for SimplePosMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option<OptValue>>>
+    PosMutCallback for SimplePosMutCallback<'a, T>
 {
     fn call(
         &mut self,
@@ -367,29 +377,30 @@ impl<T: 'static + FnMut(Uid, &mut dyn Set, &str, u64, OptValue) -> Result<Option
 }
 
 pub struct SimpleMainCallback<
-    T: 'static + FnMut(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>,
->(T);
+    'a,
+    T: 'a + Fn(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>,
+>(T, PhantomData<&'a T>);
 
-impl<T: 'static + FnMut(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>>
-    SimpleMainCallback<T>
+impl<'a, T: 'a + Fn(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>>
+    SimpleMainCallback<'a, T>
 {
     pub fn new(cb: T) -> Self {
-        Self(cb)
+        Self(cb, PhantomData::default())
     }
 }
 
-impl<T: 'static + FnMut(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>> Debug
-    for SimpleMainCallback<T>
+impl<'a, T: 'a + Fn(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>> Debug
+    for SimpleMainCallback<'a, T>
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("SimpleMainCallback")
-            .field("FnMut", &String::from("..."))
+            .field("Fn", &String::from("..."))
             .finish()
     }
 }
 
-impl<T: 'static + FnMut(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>> MainCallback
-    for SimpleMainCallback<T>
+impl<'a, T: 'a + Fn(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>> MainCallback
+    for SimpleMainCallback<'a, T>
 {
     fn call(
         &mut self,
@@ -403,19 +414,20 @@ impl<T: 'static + FnMut(Uid, &dyn Set, &[&str], OptValue) -> Result<Option<OptVa
 }
 
 pub struct SimpleMainMutCallback<
-    T: 'static + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>,
->(T);
+    'a,
+    T: 'a + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>,
+>(T, PhantomData<&'a T>);
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>>
-    SimpleMainMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>>
+    SimpleMainMutCallback<'a, T>
 {
     pub fn new(cb: T) -> Self {
-        Self(cb)
+        Self(cb, PhantomData::default())
     }
 }
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>> Debug
-    for SimpleMainMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>> Debug
+    for SimpleMainMutCallback<'a, T>
 {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("SimpleMainMutCallback")
@@ -424,8 +436,8 @@ impl<T: 'static + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<O
     }
 }
 
-impl<T: 'static + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>>
-    MainMutCallback for SimpleMainMutCallback<T>
+impl<'a, T: 'a + FnMut(Uid, &mut dyn Set, &[&str], OptValue) -> Result<Option<OptValue>>>
+    MainMutCallback for SimpleMainMutCallback<'a, T>
 {
     fn call(
         &mut self,
