@@ -1,15 +1,13 @@
 use std::convert::{TryFrom, TryInto};
+use ustr::Ustr;
 
 use super::NonOpt;
-use crate::err::{ConstructError, Error, SpecialError};
+use crate::err::Error;
+use crate::gstr;
 use crate::opt::*;
-use crate::set::{CreateInfo, Creator};
+use crate::set::CreateInfo;
+use crate::set::Creator;
 use crate::uid::Uid;
-use crate::Ustr;
-
-pub fn current_type() -> Ustr {
-    Ustr::from("p")
-}
 
 pub trait Pos: NonOpt {}
 
@@ -57,7 +55,7 @@ impl NonOpt for PosOpt {}
 
 impl Type for PosOpt {
     fn get_type_name(&self) -> Ustr {
-        current_type()
+        PosCreator::type_name()
     }
 
     fn is_deactivate_style(&self) -> bool {
@@ -73,7 +71,7 @@ impl Type for PosOpt {
 
     fn check(&self) -> Result<()> {
         if !(self.get_optional() || self.has_value()) {
-            Err(SpecialError::OptionForceRequired(self.get_hint().to_owned()).into())
+            Err(Error::sp_option_force_require(self.get_hint()))
         } else {
             Ok(())
         }
@@ -124,7 +122,7 @@ impl Name for PosOpt {
     }
 
     fn get_prefix(&self) -> Ustr {
-        Ustr::from("")
+        gstr("")
     }
 
     fn set_name(&mut self, string: Ustr) {
@@ -240,9 +238,15 @@ impl Help for PosOpt {
 #[derive(Debug, Default, Clone)]
 pub struct PosCreator;
 
+impl PosCreator {
+    pub fn type_name() -> Ustr {
+        gstr("p")
+    }
+}
+
 impl Creator for PosCreator {
     fn get_type_name(&self) -> Ustr {
-        current_type()
+        PosCreator::type_name()
     }
 
     fn is_support_deactivate_style(&self) -> bool {
@@ -252,16 +256,13 @@ impl Creator for PosCreator {
     fn create_with(&self, create_info: CreateInfo) -> Result<Box<dyn Opt>> {
         if create_info.get_support_deactivate_style() {
             if !self.is_support_deactivate_style() {
-                return Err(ConstructError::NotSupportDeactivateStyle(
-                    create_info.get_name().to_owned(),
-                )
-                .into());
+                return Err(Error::opt_unsupport_deactivate_style(
+                    create_info.get_name(),
+                ));
             }
         }
         if create_info.get_index().is_null() {
-            return Err(
-                ConstructError::MissingNonOptionIndex(create_info.get_name().to_owned()).into(),
-            );
+            return Err(Error::opt_missing_index(create_info.get_name()));
         }
 
         assert_eq!(create_info.get_type_name(), self.get_type_name());
@@ -281,17 +282,17 @@ mod test {
     fn make_type_pos_work() {
         let creator = PosCreator::default();
 
-        assert_eq!(creator.get_type_name(), current_type());
+        assert_eq!(creator.get_type_name(), PosCreator::type_name());
         // pos not support deactivate style
         assert_eq!(creator.is_support_deactivate_style(), false);
 
-        let mut ci = CreateInfo::parse(Ustr::from("pos=p@1"), &[]).unwrap();
+        let mut ci = CreateInfo::parse(gstr("pos=p@1"), &[]).unwrap();
 
         ci.set_uid(1);
 
         let mut pos = creator.create_with(ci).unwrap();
 
-        assert_eq!(pos.get_type_name(), current_type());
+        assert_eq!(pos.get_type_name(), PosCreator::type_name());
         assert_eq!(pos.is_deactivate_style(), false);
         assert_eq!(pos.match_style(Style::Pos), true);
         assert_eq!(pos.check().is_err(), false);
@@ -322,17 +323,17 @@ mod test {
         assert_eq!(pos.get_index(), Some(&OptIndex::forward(3)));
         assert_eq!(pos.match_index(6, 9), false);
 
-        assert_eq!(pos.get_name(), Ustr::from("pos"));
-        assert_eq!(pos.get_prefix(), Ustr::from(""));
+        assert_eq!(pos.get_name(), gstr("pos"));
+        assert_eq!(pos.get_prefix(), gstr(""));
         assert_eq!(pos.match_name("www".into()), true);
         assert_eq!(pos.match_name("pos".into()), true);
         assert_eq!(pos.match_prefix("--".into()), false);
         assert_eq!(pos.match_prefix("".into()), false);
-        pos.set_name(Ustr::from("pos1"));
-        pos.set_prefix(Ustr::from("+"));
+        pos.set_name(gstr("pos1"));
+        pos.set_prefix(gstr("+"));
         assert_eq!(pos.match_name("www".into()), true);
         assert_eq!(pos.match_name("pos1".into()), true);
-        assert_eq!(pos.get_name(), Ustr::from("pos1"));
+        assert_eq!(pos.get_name(), gstr("pos1"));
         assert_eq!(pos.match_prefix("+".into()), false);
         assert_eq!(pos.match_prefix("".into()), false);
 
