@@ -1,21 +1,18 @@
 use std::mem::take;
+use ustr::Ustr;
 
-use crate::err::SpecialError;
 use crate::opt::*;
-use crate::set::{CreateInfo, Creator};
+use crate::set::CreateInfo;
+use crate::set::Creator;
 use crate::uid::Uid;
-use crate::Ustr;
 
 pub mod path {
     use std::convert::{TryFrom, TryInto};
 
-    use crate::err::{ConstructError, Error, ParserError};
+    use crate::err::Error;
 
     use super::*;
-
-    pub fn current_type() -> Ustr {
-        Ustr::from("p")
-    }
+    use crate::gstr;
 
     pub trait Path: Opt {}
 
@@ -46,10 +43,9 @@ pub mod path {
         fn try_from(value: CreateInfo) -> Result<Self> {
             let mut ci = value;
             let help_info = HelpInfo::from(&mut ci);
-            let prefix = ci.get_prefix().ok_or(ConstructError::MissingOptionPrefix(
-                format!("{}", ci.get_name()),
-                format!("{}", ci.get_type_name()),
-            ))?;
+            let prefix = ci
+                .get_prefix()
+                .ok_or(Error::opt_missing_prefix(ci.get_name(), ci.get_type_name()))?;
 
             Ok(Self {
                 uid: ci.get_uid(),
@@ -71,7 +67,7 @@ pub mod path {
 
     impl Type for PathOpt {
         fn get_type_name(&self) -> Ustr {
-            current_type()
+            PathCreator::type_name()
         }
 
         fn is_deactivate_style(&self) -> bool {
@@ -87,7 +83,7 @@ pub mod path {
 
         fn check(&self) -> Result<()> {
             if !(self.get_optional() || self.has_value()) {
-                Err(SpecialError::OptionForceRequired(self.get_hint().to_owned()).into())
+                Err(Error::sp_option_force_require(self.get_hint()))
             } else {
                 Ok(())
             }
@@ -131,11 +127,10 @@ pub mod path {
         fn set_callback_ret(&mut self, ret: Option<OptValue>) -> Result<()> {
             if let Some(ret) = ret {
                 if !ret.is_any() {
-                    return Err(ParserError::InvalidReturnValueOfCallback(format!(
-                        "excepted OptValue::vec, found {:?}",
+                    return Err(Error::opt_invalid_ret_value(format!(
+                        "excepted OptValue::Vec, found {:?}",
                         ret
-                    ))
-                    .into());
+                    )));
                 }
                 self.set_value(ret);
             }
@@ -273,9 +268,15 @@ pub mod path {
     #[derive(Debug, Default, Clone)]
     pub struct PathCreator;
 
+    impl PathCreator {
+        pub fn type_name() -> Ustr {
+            gstr("p")
+        }
+    }
+
     impl Creator for PathCreator {
         fn get_type_name(&self) -> Ustr {
-            current_type()
+            PathCreator::type_name()
         }
 
         fn is_support_deactivate_style(&self) -> bool {
@@ -285,10 +286,9 @@ pub mod path {
         fn create_with(&self, create_info: CreateInfo) -> Result<Box<dyn Opt>> {
             if create_info.get_support_deactivate_style() {
                 if !self.is_support_deactivate_style() {
-                    return Err(ConstructError::NotSupportDeactivateStyle(
-                        create_info.get_name().to_owned(),
-                    )
-                    .into());
+                    return Err(Error::opt_unsupport_deactivate_style(
+                        create_info.get_name(),
+                    ));
                 }
             }
 
