@@ -5,6 +5,8 @@ use std::fmt::Debug;
 
 use crate::ctx::Context;
 use crate::err::Result;
+use crate::opt::Style;
+use crate::parser::ValueKeeper;
 use crate::set::Set;
 use crate::uid::Uid;
 
@@ -16,11 +18,16 @@ pub trait Info: Debug {
 }
 
 pub trait Proc<M: Matcher>: Debug {
-    fn process(&mut self, matcher: &mut M, set: &mut dyn Set) -> Result<bool>;
+    fn process<S: Set>(
+        &mut self,
+        matcher: &mut M,
+        set: &mut S,
+        invoke: bool,
+    ) -> Result<Vec<ValueKeeper>>;
 }
 
 pub trait Matcher: Debug {
-    fn uid(&self) -> Uid;
+    fn get_uid(&self) -> Uid;
 
     fn add_ctx(&mut self, ctx: Box<dyn Context>);
 
@@ -28,9 +35,11 @@ pub trait Matcher: Debug {
 
     fn get_ctx_mut(&mut self, index: usize) -> Option<&mut Box<dyn Context>>;
 
+    fn get_style(&self) -> Style;
+
     fn process(&mut self, uid: Uid, set: &mut dyn Set) -> Result<Option<&mut Box<dyn Context>>>;
 
-    fn undo(&mut self);
+    fn undo(&mut self, set: &mut dyn Set);
 
     fn is_matched(&self) -> bool;
 
