@@ -25,7 +25,7 @@ impl From<Uid> for NonOptMatcher {
 }
 
 impl Matcher for NonOptMatcher {
-    fn uid(&self) -> Uid {
+    fn get_uid(&self) -> Uid {
         self.uid
     }
 
@@ -49,6 +49,10 @@ impl Matcher for NonOptMatcher {
         }
     }
 
+    fn get_style(&self) -> Style {
+        self.context.as_ref().map_or(Style::Null, |v| v.get_style())
+    }
+
     fn process(&mut self, uid: Uid, set: &mut dyn Set) -> Result<Option<&mut Box<dyn Context>>> {
         let opt = set[uid].as_mut();
 
@@ -72,9 +76,11 @@ impl Matcher for NonOptMatcher {
         Ok(None)
     }
 
-    fn undo(&mut self) {
+    fn undo(&mut self, set: &mut dyn Set) {
         if let Some(ctx) = self.context.as_mut() {
-            ctx.undo();
+            if let Some(uid) = ctx.get_matched_uid() {
+                ctx.undo(set[uid].as_mut());
+            }
         }
     }
 
