@@ -2,6 +2,7 @@ use super::Matcher;
 
 use crate::ctx::Context;
 use crate::err::Result;
+use crate::opt::Style;
 use crate::set::Set;
 use crate::uid::Uid;
 
@@ -24,11 +25,15 @@ impl From<Uid> for OptMatcher {
 }
 
 impl Matcher for OptMatcher {
-    fn uid(&self) -> Uid {
+    fn get_uid(&self) -> Uid {
         self.uid
     }
 
     fn add_ctx(&mut self, ctx: Box<dyn Context>) {
+        if let Some(last) = self.context.last() {
+            // make sure the style are the same
+            assert_eq!(last.get_style(), ctx.get_style());
+        }
         self.context.push(ctx);
     }
 
@@ -40,6 +45,10 @@ impl Matcher for OptMatcher {
         self.context.get_mut(index)
     }
 
+    fn get_style(&self) -> Style {
+        self.context.last().map_or(Style::Null, |v| v.get_style())
+    }
+
     fn process(&mut self, uid: Uid, set: &mut dyn Set) -> Result<Option<&mut Box<dyn Context>>> {
         let opt = set[uid].as_mut();
 
@@ -55,8 +64,12 @@ impl Matcher for OptMatcher {
         Ok(None)
     }
 
-    fn undo(&mut self) {
-        self.context.iter_mut().for_each(|v| v.undo());
+    fn undo(&mut self, set: &mut dyn Set) {
+        self.context.iter_mut().for_each(|v| {
+            if let Some(uid) = v.get_matched_uid() {
+                v.undo(set[uid].as_mut())
+            }
+        });
     }
 
     fn is_matched(&self) -> bool {
